//********************************************************************
//*** POSIX/timer.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: timer.cc,v 1.9 2008/01/27 16:42:27 mws Exp $
//********************************************************************

#ifndef POSIX_TIMER_HH
#include "timer.hh"
#endif

#ifndef INCLUDED_ERRNO_H
#include "Exceptions/Errno.hh"
#endif

#include <cmath>
#include <iomanip>
#include <iostream>

#ifndef HAVE_GETTIMEOFDAY
#include <sys/time.h>
#endif

namespace fatalmind {

using fatalmind::POSIX::timespec;

template<class TMTS, class TM>
timer<TMTS, TM>::timer(timespec<TMTS>& tm, bool startnow)
: _start()
, _tm(tm)
, _running(startnow)
{ 
	if (_running) {
		start();
	}
}

template<class TMTS, class TM>
timer<TMTS, TM>::~timer()
{
	if (_running) {
		try {
			stop();
		} catch (...) {
			// exceptions in destructors are bad.
		}
	} 
}

template<class TMTS, class TM>
void timer<TMTS, TM>::start() {
	getRealTime(&_start);
	_running = true;
}

template<class TMTS, class TM>
void timer<TMTS, TM>::stop() {
	timespec<TM> stop;
	getRealTime(&stop);
	stop-=_start;
	
	if (stop < 0) {
		stop+=86400;
	}
	_tm+= stop;
	_running = false;
}

/*
std::ostream& operator<<(std::ostream& os, const timer& ts)
{
	char fill = os.fill();
	return os << ts.tv_sec 
			// TODO: locale
			<< '.' << std::setw(9) << std::setfill('0') << ts.tv_nsec 
			<< std::setfill(fill);
}
*/


template<class TMTS, class TM>
void timer<TMTS, TM>::getRealTime(POSIX::timespec<TM>* tm) {
#ifdef HAVE_CLOCK_GETTIME
	if (clock_gettime(CLOCK_REALTIME, tm) == -1) {
		Errno::ThrowOnError(errno);
	}
#else
	struct timeval tv;
	struct timezone tz;
	if (gettimeofday(&tv, &tz) == -1) {
		Errno::ThrowOnError(errno);
	}
	tm->tv_sec  = tv.tv_sec;
	tm->tv_nsec = tv.tv_usec * 1000;
#endif
} 

} // namespace fatalmind

