//********************************************************************
//*** ResourcePool/SQL/SQLExecute.t.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLExecute.t.cc,v 1.14 2008/01/27 16:42:28 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

using namespace fatalmind;

void test_SQLExecute_init(RP& p) {
	init(p);
}

void test_SQLExecute_one(RP& p)
{
	RP::Command::SQLExecute cmd(p, "insert into hirsch (id) values (?)");

	cmd.bind(0, 4);
	p.execute(cmd);
}

void test_SQLExecute_two(RP& p)
{
	RP::Command::SQLExecute cmd(p, "insert into hirsch (id, ukn) values (?, ?)");

	cmd.bind(0, 5);
	cmd.bind<std::string>(1, "hirsch");
	p.execute(cmd);
}

void test_SQLExecute_three(RP& p)
{
	RP::Command::SQLExecute cmd(p, "insert into hirsch (id, ukn) values (?, ?)");

	cmd.bind(0, 6);
	cmd.bind<std::string>(1, "hirsch");
	RP::PoolType::resource_t& r1 = p.get();
	cmd.execute(r1);
	cmd.bind(0, 7);
	RP::PoolType::resource_t& r2 = p.get();
	cmd.execute(r2);
	p.free(r1);
	p.free(r2);
}

void test_perf_SQLExecute_insert(RP& p)
{
	clear(p);

	for (int i = 0; i < 600; ++i) {
		RP::Command::SQLExecute inscmd(p, "insert into hirsch (id, ukn) values (?, ?)");
		inscmd.bind(0, i);
		inscmd.bind<std::string>(1, "hohoho ' kasdfjs \" sdf ");
		p.execute(inscmd);
		assertNotGrowing(i);
	}
}

void test_perf_SQLExecute_insert_outer(RP& p)
{
	clear(p);

	RP::Command::SQLExecute inscmd(p, "insert into hirsch (id, ukn) values (?, ?)");
	inscmd.bind<std::string>(1, "hohoho ' kasdfjs \" sdf ");
	for (int i = 0; i < 600; ++i) {
		inscmd.bind(0, i);
		p.execute(inscmd);
		assertNotGrowing(i);
	}
}

void test_perf_SQLExecute_update_outer(RP& p)
{
	RP::Command::SQLExecute inscmd(p, "update hirsch set ukn = ? where id = ?");
	inscmd.bind<std::string>(0, "updated");
	for (int i = 0; i < 600; ++i) {
		inscmd.bind(1, i);
		p.execute(inscmd);
		assertNotGrowing(i);
	}
}

void test_SQLExecute_fail(RP& p) {
	clear(p);

	RP::Command::SQLExecute cmd(p, "insert into hirsch (id, ukn) values (?, ?)");
	cmd.bind(0, 4);
	cmd.bind<std::string>(1, "rudi");

	p.execute(cmd);

	try {
		p.execute(cmd);
		assertNotReach();
	} catch (...) { // TODO:: this is wrong
	}
}

void test_SQLExecute_clone(RP&p) {
	clear(p);
	RP::Command::SQLExecute cmd(p, "insert into hirsch (id, ukn) values (?, ?)");
	cmd.bind<std::string>(1, "rudi2");
	std::auto_ptr<RP::Command::SQLExecute> cmd2(cmd.clone<RP::Command::SQLExecute>());
	cmd.bind(0, 4);
	cmd2->bind(0, 5);

	p.execute(cmd);
	p.execute(*cmd2);

	cmd2->bind(0,6);
	p.execute(*cmd2);
	try {
		p.execute(cmd);
		assertNotReach();
	} catch (...) {
	}
}


